#!/usr/bin/perl -w
use strict;

# Path to CURL to cmakify.
my $cpath = '../curl-7.19.4';

# 1 apply md5 patch to lib/md5.c
# 2 check if all source files are included in the list.
# 3. check if all header files are checked for
# 4. compare the list of checks (possibly from m4 config script) - TBD.

if($^O eq 'linux') # Works or linux only.
{
    my $cwd = `pwd`;
    chomp $cwd;
    chdir $cpath;

    system("cp $cwd/lib-lists.txt lib/CMakeLists.txt");
    system("cp $cwd/inc-lists.txt include/CMakeLists.txt");
    system("cp $cwd/inc-curl-lists.txt include/curl/CMakeLists.txt");
    
    system("patch -p1 <$cwd/md5.patch");
    mkdir "CMake", 0777;
    system("cp -f $cwd/CMake/* CMake/");
    mkdir "Platforms", 0777;
    system("cp -f $cwd/Platforms/* Platforms/");
    chdir $cwd;
}
else
{
    die("Works on Linux only!");
}

my %noconv = ( inline => 1, _ALL_SOURCE => 1 );

my %strconv = ( RANDOM_FILE => 1 );

open HI, "<$cpath/lib/config.h.in" or die $!;
open HO, ">$cpath/lib/config.h.cmake" or die $!;

my $line;
while($line = <HI>)
{
	chomp $line;
	if($line =~ /^#undef\s+(\S+)\s?$/)
	{
		if(!defined($noconv{$1}))
		{
		    if(defined($strconv{$1}))
		    {
			$line = "#cmakedefine $1 \"\${$1}\"";
		    }
		    else
		    {
			$line = "#cmakedefine $1 \${$1}";
		    }
		}
	}
	print HO "$line\n";
}

close HI;
close HO;

open HI, "<$cpath/include/curl/curlbuild.h.in" or die $!;
open HO, ">$cpath/include/curl/curlbuild.h.cmake" or die $!;

while($line = <HI>)
{
	chomp $line;
	if($line =~ /^#undef\s+(\S+)\s?$/)
	{
		if(!defined($noconv{$1}))
		{
		    if(defined($strconv{$1}))
		    {
			$line = "#cmakedefine $1 \"\${$1}\"";
		    }
		    else
		    {
			$line = "#cmakedefine $1 \${$1}";
		    }
		}
	}
	print HO "$line\n";
}

close HI;
close HO;

# Get list of header files.

my %headers = ();
my $inhdr = 0;
my ($i, @a);

open HI, "<$cpath/configure.ac" or die $!;
my $cline = '';
while($line = <HI>)
{
    chomp $line;
    $_ = $line;
    if(/^\s*AC_CHECK_HEADERS\(/ .. /[\,\)]/)
    {
	$cline .= " $line";
    }
}
close HI;
open HI, "<$cpath/m4/curl-functions.m4" or die $!;
while($line = <HI>)
{
    chomp $line;
    $_ = $line;
    if(/^\s*AC_CHECK_HEADERS\(/ .. /[\,\)]/)
    {
	$cline .= " $line";
    }
}
close HI;
open HI, "<$cpath/acinclude.m4" or die $!;
while($line = <HI>)
{
    chomp $line;
    $_ = $line;
    if(/^\s*AC_CHECK_HEADERS\(/ .. /[\,\)]/)
    {
	$cline .= " $line";
    }
}
close HI;

$cline =~ s/[\(\)\[\]\,\\]/ /g;
$cline =~ s/AC_CHECK_HEADERS/ /g;
$cline =~ s/\s+/ /g;
$cline =~ s/^\s+//;
$cline =~ s/\s+$//;
@a = split(/ /, $cline);
for $i (@a)
{
    $headers{$i} = 1;
}

# Now derive the variables for headers.
for $i (keys %headers)
{
    $inhdr = uc $i;
    $inhdr =~ s/\./_/g;
    $inhdr =~ s/\//_/g;
    $inhdr =~ s/\-/_/g;
    $headers{$i} = 'HAVE_' . $inhdr;
}

# Dump Cmake config script.
open HI, "<CMakeLists.curl" or die $!;
open HO, ">$cpath/CMakeLists.txt" or die $!;

while($line = <HI>)
{
	chomp $line;
	if($line eq '@HEADERS@')
	{
	    for $i (sort keys %headers)
	    {
		if(($i =~ /sys/) || ($i =~ /type/))
		{
		# CHECK_INCLUDE_FILE_CONCAT("process.h"        HAVE_PROCESS_H)
		print HO sprintf("CHECK_INCLUDE_FILE_CONCAT(\"%s\" %21s)\n", $i, $headers{$i});
		}
	    }
	    for $i (sort keys %headers)
	    {
		if(($i !~ /sys/) && ($i !~ /type/))
		{
		    print HO sprintf("CHECK_INCLUDE_FILE_CONCAT(\"%s\" %21s)\n", $i, $headers{$i});
		}
	    }
	}
	else
	{
	    print HO "$line\n";
	}
}

close HI;
close HO;

